use crate::indicators::git::{repo_name, MAIN_COL};
use ansi_term::ANSIGenericString;
use ansi_term::Colour::RGB;
use std::env::current_dir;

pub const UNKNOWN_PATH: &str = "\u{2248}"; // "≈"
pub const PATH_COL: ansi_term::Colour = RGB(82, 82, 82);
pub const REPO_COL: ansi_term::Colour = RGB(55, 120, 130);

fn home_dir() -> String {
    std::env::var("HOME").map_or_else(|_| "".to_string(), |path| path.to_string())
}

fn full_path() -> String {
    current_dir().map_or_else(
        |_| UNKNOWN_PATH.to_string(),
        |path| path.to_string_lossy().to_string(),
    )
}

fn remove_repo(pwd_path: &str, repo_path: &str) -> String {
    pwd_path.replacen(repo_path, "", 1)
}

fn replace_home(path: &str) -> String {
    let homedir = home_dir();
    path.replacen(&homedir, "~", 1)
}

fn short(path: &str, slash_limit: u8) -> String {
    let slashes = path.matches('/').count();
    if slashes <= slash_limit.into() {
        path.to_string() // Short path, return without changes
    } else {
        // Long path, shorten it
        let parts: Vec<&str> = path.split('/').collect(); // Split the path into parts
        let first = if path.starts_with("~") { "~" } else { "" }; // Determine the first part correctly
        let last = parts[parts.len() - 1]; // The last part

        // Abbreviate middle parts (take the first character of each)
        let abbreviated_middle: Vec<String> = parts[1..parts.len() - 1] // Skip the first and last part
            .iter()
            .filter(|&&part| !part.is_empty()) // Avoid empty parts (like after "/")
            .map(|&part| part.chars().next().unwrap().to_string()) // Take the first letter
            .collect();

        // Join the parts back together with "/" and return the shortened path
        let mut shortened_path = vec![first.to_string()];
        shortened_path.extend(abbreviated_middle);
        shortened_path.push(last.to_string());

        shortened_path.join("/")
    }
}

pub fn pwd(
    abbrev_home: bool,
    shorten_path: bool,
    replace_repo: bool,
    git_repo: Option<String>,
) -> ANSIGenericString<'static, str> {
    let mut path = full_path();
    let slash_limit: u8 = if replace_repo { 2 } else { 3 };

    if replace_repo && git_repo.is_some() {
        if let Some(repo_path) = git_repo {
            path = remove_repo(&path, &repo_path);
            let repo_name = repo_name(&repo_path);
            if shorten_path {
                path = short(&path, slash_limit);
            }
            match path.as_str() {
                "" => REPO_COL.paint(repo_name),
                _ => format!(
                    "{}{}{}",
                    REPO_COL.paint(repo_name),
                    MAIN_COL.paint(" \u{F02A2} "),
                    PATH_COL.italic().paint(path)
                )
                .into(),
            }
        } else {
            PATH_COL.italic().paint(path)
        }
    } else if abbrev_home {
        path = replace_home(&path);
        if shorten_path {
            path = short(&path, slash_limit);
        }
        PATH_COL.italic().paint(path)
    } else {
        PATH_COL.italic().paint(path)
    }
}
